package ru.amse.smyshlyaev.grapheditor.graph;

import java.util.*;

import ru.amse.smyshlyaev.grapheditor.starvertex.*;


public class Graph implements IGraph {
	private final Set<ISelectableVertex> myVertices;
	
	public Graph() {
		myVertices = new HashSet<ISelectableVertex>();
	}
	
	private boolean hasEdgeStartAndEnd(IEdge edge) {
		return myVertices.contains(edge.getStart()) && 
        	myVertices.contains(edge.getEnd());
	}

	public static Graph unmodifiableGraph(final Graph graph) {
		return new Graph() {
			public boolean addVertex(ISelectableVertex vertex) {
				throw new UnsupportedOperationException();				
			}
			
			public boolean addVertices(Set<ISelectableVertex> vertex) {
				throw new UnsupportedOperationException();
			}
			
			public boolean removeVertex(ISelectableVertex vertex) {
				return graph.removeVertex(vertex);
			}
			
			public boolean removeVertices(List<ISelectableVertex> list) {
				throw new UnsupportedOperationException();
			}

			public Set<ISelectableVertex> getVertices() {
				return graph.getVertices();
			}
						
			public boolean removeEdge(IEdge edge) {
				return graph.removeEdge(edge);
			}
			
			public boolean hasVertex(ISelectableVertex vertex) {
				return graph.hasVertex(vertex);
			}

			public boolean hasEdge(IEdge edge) {
				return graph.hasEdge(edge);
			}

			public boolean addEdge(IEdge edge) {
				throw new UnsupportedOperationException();
			}
		};
	}
	
	public boolean addEdge(IEdge edge) {
		if (!hasEdgeStartAndEnd(edge)) {
			return false;
		}
		
		edge.getStart().addEdge(edge);
		edge.getEnd().addEdge(edge);		
		
		return true;
	}

	public boolean addVertex(ISelectableVertex vertex) {
		return myVertices.add(vertex);
	}

	public boolean addVertices(Set<ISelectableVertex> set) {
		return myVertices.addAll(set);
	}
	
	public Set<ISelectableVertex> getVertices() {
		return Collections.unmodifiableSet(myVertices);
	}

	public boolean removeEdge(IEdge edge) {
		if (!hasEdgeStartAndEnd(edge)) {
			return false;
		}
		
		edge.getStart().removeEdge(edge);
		edge.getEnd().removeEdge(edge);
		
		return true;
	}

	public boolean removeVertex(ISelectableVertex vertex) {
		myVertices.remove(vertex);
		
		if (vertex.isParent()) {
			List<Child> children = 
				((StarVertex) vertex).getChildren();
			
			for (Child child : children) {
				removeVertex(child);
			}
		}
		
		if (vertex.isChild()
				&& hasVertex(((Child) vertex).getFather())){
			((Child) vertex).getFather().removeChild(
					((Child) vertex)
			);
		}
		
		return true;
	}
	
	public boolean removeVertices(List<ISelectableVertex> list) {
/*		try {
		for (ISelectableVertex v : set) {
		for (int i = 0; i < list.size(); ++i) {
			removeVertex(list.get(i));

			if (v.isChildrenCreator()) {
				myVertices.removeAll(
						((StarVertex) v).getChildren()
				);
			}
		}
		} catch (ConcurrentModificationException e) {
			e.printStackTrace();
		}
*/		
		boolean returnValue = true;
		
		for (ISelectableVertex v : list) {
			if (!removeVertex(v)) {
				returnValue = false;
			}
		}
		
		return returnValue;
	}
	
	public boolean hasVertex(ISelectableVertex vertex) {
		return myVertices.contains(vertex);
	}
	
	public boolean hasEdge(IEdge edge) {
		return myVertices.contains(edge.getStart()) &&
    		myVertices.contains(edge.getEnd()) &&
	    	edge.getStart().hasEdge(edge) &&
    		edge.getStart().hasEdge(edge);
	}
}