package ru.amse.smyshlyaev.grapheditor.graph;

import java.util.*;
import java.awt.*;

import ru.amse.smyshlyaev.grapheditor.graphdatacalculator.VertexDataCalculator;


public class Vertex extends LabelOwner implements ISelectableVertex {
	private final Set<IEdge> myEdges;
	protected int myX;
	protected int myY;
	public static final int MY_RADIUS = 20;
	public static final int MY_SPACE = 10;
		
	public Vertex(int x, int y, String message) {
		super(message);

		myEdges = new HashSet<IEdge>();
		myX = x;
		myY = y;
	}

	public Vertex(int x, int y) {
		this(x, y, "");
	}

	public Vertex() {
		this(0, 0, "");
	}
	
	public int getX() {
		return myX;
	}
	
	public void addLocation(int x, int y) {
		myX += x;
		myY += y;
	}
	
	public void subLocation(int x, int y) {
		myX -= x;
		myY -= y;
	}

	public void setLocation(int x, int y) {
		myX = x;
		myY = y;
	}
	
	public int getY() {
		return myY;
	}

	public Set<IEdge> getEdges() {
		return Collections.unmodifiableSet(myEdges);
	}
	
	public int getEdgesSize() {
		return myEdges.size();
	}
	
	public boolean addEdge(IEdge edge) {
		if ((edge == null) 
				|| hasEdge(edge)) {
			return false;
		}

		return myEdges.add(edge);
	}
	
	public boolean removeEdge(IEdge edge) {
		return myEdges.remove(edge);
	}
	
	public boolean hasEdge(IEdge edge) {
		if (edge == null)  {
			return false;
		}
		
		for (IEdge e : myEdges) {
			if (e.getStart() != null 
					&& e.getEnd() != null
					&& e.getStart().equals(edge.getStart())
					&& e.getEnd().equals(edge.getEnd()))
				return true;
		}
		
		return false;
	}
	
	public void select() {
	}
	
	public void deselect() {
	}

	public Rectangle getSelectionRectangle(Graphics gc) {
		final int SPACE = 10;
		
		int width = 
			VertexDataCalculator.getWidth(gc, this) + 
	    		SPACE;
        int height = 
        	VertexDataCalculator.getHeight(gc, this) + 
            	SPACE;

		return new Rectangle(
				getX() - width / 2, 
		    	getY() - height / 2,
	    		width,
	   			height
	   	);
	}
	
	public boolean shouldSelectAlone() {
		return false;
	}
	
	public int getHorizontalRadius(Graphics gc) {
		return VertexDataCalculator.getWidth(gc, this) / 2;
	}

	public int getVerticalRadius(Graphics gc) {
		return VertexDataCalculator.getHeight(gc, this) / 2;
	}
	
	public boolean isParent() {
		return false;
	}
	
	public boolean isChild() {
		return false;
	}
}