package ru.amse.smyshlyaev.grapheditor.io;

import java.util.*;
import java.io.*;
import javax.xml.stream.*;
import javax.xml.parsers.*;

import org.xml.sax.Attributes;
import org.xml.sax.helpers.*;
import org.xml.sax.*;

import ru.amse.smyshlyaev.grapheditor.graph.*;
import ru.amse.smyshlyaev.grapheditor.starvertex.*;


public class GraphXMLFileReader {
	private static List<ISelectableVertex> numberedVertices;

	private static ISelectableVertex readVertex(Attributes attributes) {
		if ("general".equals(attributes.getValue(0))) {
			return new Vertex(
    					Integer.valueOf(
							attributes.getValue(2)
						),
			   			Integer.valueOf(
						    attributes.getValue(3)
			   		    ),
					    attributes.getValue(4)
			 	    );
		}
		
		if ("root".equals(attributes.getValue(0))) {			
			Root root = new Root(
    				Integer.valueOf(
						attributes.getValue(2)
					),
		   			Integer.valueOf(
					    attributes.getValue(3)
		   		    ),
				    attributes.getValue(4)
		 	    );
		
			root.setRadius(
				Integer.valueOf(
					attributes.getValue(5)
				)
	    	);
		
    		root.setStartAngle(
	 			Double.valueOf(
					attributes.getValue(6)
				)
	    	);

    		root.setSpace(
				Integer.valueOf(
					attributes.getValue(7)
				)
	    	);

			return root;
		}
		
		Child child = new Child(
					attributes.getValue(2),
		   			Integer.valueOf(
					    attributes.getValue(7)
		   		    ),
				    (StarVertex) numberedVertices.get(
				    	Integer.valueOf(
				    		attributes.getValue(5)
				    	)
				    )
				);

		child.setRadius(
			Integer.valueOf(
				attributes.getValue(3)
			)
		);
		
		child.setStartAngle(
			Double.valueOf(
				attributes.getValue(4)
			)
		);
		
		child.setSpace(
			Integer.valueOf(
				attributes.getValue(6)
			)
		);
		
		return child;
	}
	
	private static IEdge readEdge(Attributes attributes,
	          		List<ISelectableVertex> numberedVertices) {
		if (Integer.valueOf(attributes.getValue(1)) == -1) {
			return null;
		}
		
		return new Edge(
			numberedVertices.get(
				Integer.valueOf(
	   		    	attributes.getValue(0)
   	        	)
	        ),
			numberedVertices.get(
				Integer.valueOf(
   				    attributes.getValue(1)
                )
	        ),
	        attributes.getValue(2)
		);
	}
	
	public static Graph readGraph(File file) {
		FileReader fileReader = null;
		XMLStreamReader xmlReader = null;
		final Graph graph = new Graph();

		numberedVertices = 
			new ArrayList<ISelectableVertex>();
		 
		 try {
			SAXParserFactory factory =
				SAXParserFactory.newInstance();
			
			SAXParser parser = factory.newSAXParser();
			
			parser.parse(
				file,
				new DefaultHandler() {
					public void startElement(String uri, 
							String localName, String name, 
							Attributes attributes) {
						if (name.equals("vertex")) {

							ISelectableVertex vertex = readVertex(attributes);

							graph.addVertex(vertex);						
							numberedVertices.add(vertex);						

						} else if (name.equals("edge")) {
							IEdge edge = readEdge(
		 	        	    		attributes,
		 	            			numberedVertices
		 		        	    );
							if (edge != null) {
						 		graph.addEdge(edge);
							}
						}
					}
				}
			);			
		} catch (IOException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} finally {
			Closer.closeXML(xmlReader);
			Closer.close(fileReader);
		}

		return graph;
	}
}