package ru.amse.smyshlyaev.grapheditor.io;

import java.io.*;
import java.util.*;
import javax.xml.stream.*;

import ru.amse.smyshlyaev.grapheditor.graph.*;
import ru.amse.smyshlyaev.grapheditor.starvertex.*;


public class GraphXMLFileWriter {
    private static int vertexIndex;
    
	private static List<ISelectableVertex> writeChild(XMLStreamWriter xmlWriter,
			Child child, List<ISelectableVertex> numberedVertices) throws XMLStreamException {
        xmlWriter.writeStartElement("vertex");

        xmlWriter.writeAttribute("type", "child");
        xmlWriter.writeAttribute("index", "" + vertexIndex);
        xmlWriter.writeAttribute("label", child.getLabel());
        xmlWriter.writeAttribute("radius", "" + child.getRadius());
        xmlWriter.writeAttribute("angle", "" + child.getStartAngle());
        xmlWriter.writeAttribute("father", "" + numberedVertices.indexOf(child.getFather()));
        xmlWriter.writeAttribute("space", "" + child.getSpace());
        xmlWriter.writeAttribute("child_index", "" + child.getIndex());

        vertexIndex += 1;
        
        numberedVertices.add(child);
        
        xmlWriter.writeEndElement();
         
        if (!child.getChildren().isEmpty()) {
        	for (Child v : child.getChildren()) {
    			writeChild(
        				xmlWriter,
        				v,
        				numberedVertices
    			);
        	}
        }

        return numberedVertices;
	}
	
	private static List<ISelectableVertex> writeVertices(XMLStreamWriter xmlWriter,
        	Set<ISelectableVertex> vertices) throws XMLStreamException {
		if ((xmlWriter == null) 
				|| (vertices == null)) {
			return null;
		}

		List<ISelectableVertex> numberedVertices = 
	    		new ArrayList<ISelectableVertex>();
        
        for (ISelectableVertex v : vertices) {
            if (v instanceof Root) {
                xmlWriter.writeStartElement("vertex");
                
                xmlWriter.writeAttribute("type", "root");
                xmlWriter.writeAttribute("index", "" + vertexIndex);
                xmlWriter.writeAttribute("x", "" + v.getX());
                xmlWriter.writeAttribute("y", "" + v.getY());
                xmlWriter.writeAttribute("label", v.getLabel());
                xmlWriter.writeAttribute("radius", "" + ((Root) v).getRadius());
                xmlWriter.writeAttribute("angle", "" + ((Root) v).getStartAngle());
                xmlWriter.writeAttribute("space", "" + ((Root) v).getSpace());
                numberedVertices.add(v);
                vertexIndex += 1;			                
                
                xmlWriter.writeEndElement();
                    
                if (!((Root) v).getChildren().isEmpty()) {
                	for (Child child : ((Root) v).getChildren()) {
            			writeChild(
                				xmlWriter,
                				child,
                				numberedVertices
            			);
                	}
                }
            } else if (!(v instanceof StarVertex)){
                xmlWriter.writeStartElement("vertex");
                
                xmlWriter.writeAttribute("type", "general");
                xmlWriter.writeAttribute("index", "" + vertexIndex);
                xmlWriter.writeAttribute("x", "" + v.getX());
                xmlWriter.writeAttribute("y", "" + v.getY());
                xmlWriter.writeAttribute("label", v.getLabel());	
                numberedVertices.add(v);
                vertexIndex += 1;			                

                xmlWriter.writeEndElement();
            }
        }
        
        return numberedVertices;
	}
	
	private static void writeEdges(XMLStreamWriter xmlWriter,
	        		List<ISelectableVertex> numberedVertices)
                    throws XMLStreamException {
		if ((xmlWriter == null) 
				|| (numberedVertices == null)) {
			return;
		}
			
        for (ISelectableVertex v : numberedVertices) {
            Set<IEdge> edges = v.getEdges();
            
            for (IEdge edge : edges) {
            	if (edge.getStart() == v) {
	                xmlWriter.writeStartElement("edge");
	                xmlWriter.writeAttribute("start", "" + numberedVertices.indexOf(edge.getStart()));
	                xmlWriter.writeAttribute("end", "" + numberedVertices.indexOf(edge.getEnd()));
	                xmlWriter.writeAttribute("label", edge.getLabel());
	                xmlWriter.writeEndElement();
            	}
            }
        }		
	}
	
	public static void writeGraph(File file, IGraph graph) 
                	throws IOException, XMLStreamException {
		if ((graph == null) || (file == null)) {
			return;
		}
		
		vertexIndex = 0;
		
		FileOutputStream outStream = null;
		XMLStreamWriter xmlWriter = null;
		
		try {
			XMLOutputFactory factory = 
				XMLOutputFactory.newInstance();
			
			outStream = new FileOutputStream(file);
			
			xmlWriter = factory.createXMLStreamWriter(
		            			outStream, "UTF-8"
                        );
			
			xmlWriter.writeStartDocument("UTF-8", "1.0");
			xmlWriter.writeStartElement("graph");
            
            List<ISelectableVertex> numberedVertices = 
                	writeVertices(
                			xmlWriter, 
                			graph.getVertices()
                	);
            
            writeEdges(xmlWriter, numberedVertices);

            xmlWriter.writeEndElement();		
            xmlWriter.writeEndDocument();
    	} finally {
    		Closer.closeXML(xmlWriter);
    		Closer.close(outStream);
    	}
	}
}