package ru.amse.smyshlyaev.grapheditor.ui;

import javax.swing.*;
import java.awt.*;
import java.io.*;
import javax.xml.stream.*;
import java.awt.event.*;

import ru.amse.smyshlyaev.grapheditor.graph.*;
import ru.amse.smyshlyaev.grapheditor.io.*;
import ru.amse.smyshlyaev.grapheditor.ui.filechooser.FileChooser;


public class GraphEditor {
	private static java.net.URL imageDefaultURL = 
		GraphEditor.class.getResource("images/default.gif");
	private static java.net.URL imageStarURL = 
		GraphEditor.class.getResource("images/star.GIF");
	private static java.net.URL imageSaveURL = 
		GraphEditor.class.getResource("images/save.gif");
	private static java.net.URL imageOpenURL = 
		GraphEditor.class.getResource("images/open.gif");
	private static java.net.URL imageVertexURL = 
		GraphEditor.class.getResource("images/vertex.GIF");
	private static java.net.URL imageEdgeURL = 
		GraphEditor.class.getResource("images/edge.GIF");
	private static JGraphComponent myGraphComponent;
	private static final int MY_FRAME_SIZE = 700;
	private static final Color MY_TOOLBAR_COLOR = 
                         		Color.getHSBColor(
	                    			(float) 0.1, 
                					(float) 0.4, 
				                   	(float) 0.9
	                    		);

	private static AbstractAction getOpenAction(final Component parent) {
		return new AbstractAction("Open") {
			public void actionPerformed(ActionEvent e) {
				File file = FileChooser.choose(
				    		parent, 
					    	"Open"
					   );
				
				if (file == null) {
					return;
				}
				
				Graph graph = GraphXMLFileReader.readGraph(
		                			file
				);

				if (graph == null) {
					return;
				}
				
				myGraphComponent.setGraph(graph);
				myGraphComponent.repaint();
			}
		};
	}
	
	private static AbstractAction getSaveAction(final Component parent) {
		return new AbstractAction("Save") {
			public void actionPerformed(ActionEvent e) {
				File file = FileChooser.choose(
				    		parent,
					    	"Save"
					   );
				
				if (file == null) {
					return;
				}
			
				try {
					GraphXMLFileWriter.writeGraph(
							file,
			                myGraphComponent.getGraph()
					);					
				} catch (IOException exception) {
					exception.printStackTrace();
				} catch (XMLStreamException exception) {
					exception.printStackTrace();
				}
			}
		};
	}
	
	private static AbstractAction getDeleteAction() {
		return new AbstractAction("Delete") {
			public void actionPerformed(ActionEvent e) {
				myGraphComponent.removeMarkedVertices();
				myGraphComponent.clearMarkedVertices();
				myGraphComponent.repaint();				
			}
		};
	}
	
	private static void setButtonProperties(AbstractButton button,
        			 Icon icon ,ActionListener action) {
		button.setPreferredSize(new Dimension(MY_FRAME_SIZE / 20, MY_FRAME_SIZE / 20));
		button.setFocusPainted(false);
		button.setBackground(MY_TOOLBAR_COLOR);
		button.addActionListener(action);
		button.setIcon(icon);
	}

	private static JToolBar getToolBar(Dimension size,
	        		Color color, LayoutManager layout,
	        		Insets insets,
	        		final JGraphComponent component) {
		JToolBar toolBar = new JToolBar();

		toolBar.setPreferredSize(size);
		toolBar.setFloatable(false);
		toolBar.setBackground(color);
		toolBar.setLayout(layout);
		toolBar.setBorderPainted(true);
		toolBar.setMargin(insets);

		return toolBar;
	}
	
	private static JToolBar getVerticalToolBar() {
		return getToolBar(
 	    			new Dimension(
	    					MY_FRAME_SIZE / 20 + 10, 
		     				MY_FRAME_SIZE
			    	),
   		    		MY_TOOLBAR_COLOR,
 			    	new FlowLayout(FlowLayout.CENTER),
	    			new Insets(10, 0, 0, 0),
		    		myGraphComponent
	        	);
	}
	
	private static JToolBar getHorizontalToolBar() {
		return getToolBar(
 	    			new Dimension(
	    					MY_FRAME_SIZE, 
		     				MY_FRAME_SIZE / 20 + 10
			    	),
   		    		MY_TOOLBAR_COLOR,
 			    	new FlowLayout(FlowLayout.LEFT),
	    			new Insets(0, 10, 0, 0),
		    		myGraphComponent
	        	);
	}

	private static JToolBar getGraphToolBar() {
		JToolBar graphToolBar = getVerticalToolBar();
			
		JToggleButton defaultToolButton = new JToggleButton();
		JToggleButton vertexCreatorButton = new JToggleButton();
		JToggleButton edgeCreatorButton = new JToggleButton();
		JToggleButton starCreatorButton = new JToggleButton();

		setButtonProperties(
				defaultToolButton, 
				new ImageIcon("images/default.gif"),
				new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						myGraphComponent.setDefaultTool();
					}
				}
		);
		
		setButtonProperties(
				vertexCreatorButton, 
				new ImageIcon("images/vertex.GIF"),
				new ActionListener() {
					public void actionPerformed(ActionEvent e) {
		    			myGraphComponent.createVertex();
					}
				}
		);
		
		setButtonProperties(
				edgeCreatorButton, 
				new ImageIcon("images/edge.GIF"),
				new ActionListener() {
					public void actionPerformed(ActionEvent e) {
	    				myGraphComponent.createEdge();
					}
				}
		);

		setButtonProperties(
				starCreatorButton, 
				new ImageIcon("images/star.GIF"),
				new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						myGraphComponent.createStar();
					}
				}
		);

		ButtonGroup buttonGroup = new ButtonGroup();
		
		buttonGroup.add(defaultToolButton);
		buttonGroup.add(vertexCreatorButton);
		buttonGroup.add(edgeCreatorButton);
		buttonGroup.add(starCreatorButton);

		defaultToolButton.setSelected(true);
		
		graphToolBar.add(defaultToolButton);
		graphToolBar.add(vertexCreatorButton);
		graphToolBar.add(edgeCreatorButton);
		graphToolBar.add(starCreatorButton);
		
		return graphToolBar;
	}
	
	private static JToolBar getMenuToolBar(final Component parent) {
		JToolBar menuToolBar = getHorizontalToolBar();
		
		JButton openButton = new JButton();
		JButton saveButton = new JButton();
		
		setButtonProperties(
				openButton, 
				new ImageIcon("images/open.gif"),
				getOpenAction(parent)
		);
		
		setButtonProperties(
				saveButton, 
				new ImageIcon("images/save.gif"),
		    	getSaveAction(parent)
		);
		
		menuToolBar.add(openButton);
		menuToolBar.add(saveButton);

		return menuToolBar;
	}
	
	private static JMenuBar getMenuBar(final Component parent) {
		JMenuBar menuBar = new JMenuBar();
		JMenu fileMenu = new JMenu("File");
		
		fileMenu.add(getOpenAction(parent));
		fileMenu.add(getSaveAction(parent));
		
		menuBar.add(fileMenu);
		
		JMenu optionsMenu = new JMenu("Options");
		
		optionsMenu.add(getDeleteAction());
		
		menuBar.add(optionsMenu);
		
		return menuBar;
	}
	
	public static void main(String[] args) {
		final JFrame frame  = new JFrame("Graph Editor");
		
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(MY_FRAME_SIZE, MY_FRAME_SIZE);

		frame.setJMenuBar(getMenuBar(frame));

		frame.getContentPane().add(
				getMenuToolBar(frame),
				BorderLayout.NORTH
		);

		frame.getContentPane().add(
				getGraphToolBar(), 
				BorderLayout.WEST
		);
				
		Graph graph = new Graph();
		myGraphComponent = new JGraphComponent(
                				graph, 
                				10000,
                				10000
                		   );

		frame.getContentPane().add(
			new JScrollPane(
				myGraphComponent,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS
			)
		);
		
		frame.setVisible(true);
	}
}