package ru.amse.smyshlyaev.grapheditor.ui.drawer;

import java.awt.*;

import ru.amse.smyshlyaev.grapheditor.graph.ISelectableVertex;
import ru.amse.smyshlyaev.grapheditor.graphdatacalculator.VertexDataCalculator;


public class VertexDrawer {
	public static final int BUTTON_WIDTH = 10;
	public static final int BUTTON_HEIGHT = 10;
	private ISelectableVertex myVertex;
	private Color myColor;
	
	public VertexDrawer(ISelectableVertex vertex) {
		myVertex = vertex;
		myColor = Color.ORANGE;
	}
	
	public VertexDrawer() {
		this(null);
	}
	
	public void setVertex(ISelectableVertex vertex) {
		myVertex = vertex;
	}
	
	public void setColor(Color color) {
		myColor = color;
	}
	
	public Color getColor() {
		return myColor;
	}
	
	private Rectangle getChildrenCreatorRectangle(Graphics gc) {
		final int Y_SPACE = 5;

		return new Rectangle(
				myVertex.getX() - BUTTON_WIDTH / 2, 
				myVertex.getY() + 
			    	VertexDataCalculator.getHeight(
			    		gc, 
			    		myVertex
			    	) / 2 - Y_SPACE - BUTTON_HEIGHT,
				BUTTON_WIDTH, 
				BUTTON_HEIGHT
		);
	}

	public void draw(Graphics gc) {
		int width = VertexDataCalculator.getWidth(gc, myVertex);
		int height = VertexDataCalculator.getHeight(gc, myVertex);

		int x = myVertex.getX() - width / 2;

		int y = myVertex.getY() - height / 2;

		Color currentColor = gc.getColor();

		gc.setColor(myColor);

		gc.fillOval(
			x, 
			y, 
			width,
			height
		);

		gc.setColor(currentColor);

		gc.drawString(
			myVertex.getLabel(),
			myVertex.getX() - 
    			VertexDataCalculator.getLabelWidth(
        			gc, 
	        		myVertex
        		) / 2,
			myVertex.getY() + 
       			VertexDataCalculator.getLabelHeight(
        			gc, 
		        	myVertex
       			) / 2
		);
		
		if (myVertex.isParent()) {
			Rectangle childrenCreator = 
				getChildrenCreatorRectangle(gc);
			
			gc.drawRect(
				childrenCreator.x, 
				childrenCreator.y, 
				childrenCreator.width, 
				childrenCreator.height
			);
		}
	}
}