package ru.amse.smyshlyaev.grapheditor.ui.tool.defaulttool;

import java.awt.event.*;
import java.util.Set;
import java.awt.Graphics;

import ru.amse.smyshlyaev.grapheditor.ui.JGraphComponent;
import ru.amse.smyshlyaev.grapheditor.graph.*;
import ru.amse.smyshlyaev.grapheditor.ui.tool.*;
import ru.amse.smyshlyaev.grapheditor.graphdatacalculator.*;
import ru.amse.smyshlyaev.grapheditor.starvertex.*;


public class DefaultToolManager extends ToolAdapter
                 implements ITool {
	private final JGraphComponent myComponent;
	private ITool myTool;
	private final DefaultRectangleTool myRectangleTool;
	private final DefaultVertexTool myVertexTool;
	private final DefaultEdgeTool myEdgeTool;

	public DefaultToolManager(JGraphComponent component) {
		myComponent = component;
		
		myRectangleTool = 
			new DefaultRectangleTool(myComponent);
		
		myVertexTool = 
			new DefaultVertexTool(myComponent);
		
		myEdgeTool = 
			new DefaultEdgeTool(myComponent);

		myTool = myVertexTool;
	}

	public void paint(Graphics gc) {
		myTool.paint(gc);
	}

	private boolean isInMarkedEdge(int x, int y) {
		IEdge e = myComponent.getMarkedEdge();
		
		if (e == null) {
			return false;
		}

		if (DefaultEdgeTool.isInRectangle(
				EdgeDataCalculator.getStartX(myComponent.getGraphics(), e),
				EdgeDataCalculator.getStartY(myComponent.getGraphics(), e),
				x,
				y
			)
		) {
			return true;
		}

		if (DefaultEdgeTool.isInRectangle(
				EdgeDataCalculator.getEndX(myComponent.getGraphics(), e),
				EdgeDataCalculator.getEndY(myComponent.getGraphics(), e),
				x, 
				y
			)
		) {
				return true;
		}
		
		return EdgeDataCalculator.isInEdge(myComponent.getGraphics(), e, x, y);
	}

	public void mousePressed(MouseEvent e) {
		int x = e.getX();
		int y = e.getY();
		
		if (isInMarkedEdge(x, y)) {
			myTool.mousePressed(e);
			return;			
		}

		Set<ISelectableVertex> vertices = 
			myComponent.getGraph().getVertices();
		
		for (ISelectableVertex v : vertices) {
			if (VertexDataCalculator.isInVertex(myComponent.getGraphics(), v, x, y)) {
				myTool = myVertexTool;
				myComponent.setMarkedEdge(null);
				myVertexTool.setVertex(v);
				myTool.mousePressed(e);
				return;
			}

			Set<IEdge> edges = v.getEdges();

	    	for (IEdge edge : edges) {
	    		if (edge.getStart() != null
	    				&& edge.getEnd() != null
	    				&& EdgeDataCalculator.isInEdge(myComponent.getGraphics(), edge, x, y) 
	    				&& (edge.getStart().equals(v))) {
                	myTool = myEdgeTool;
                	myComponent.clearMarkedVertices();
                	myComponent.setMarkedEdge(edge);
                	myTool.mousePressed(e);
                 	return;
	    		}
	    	}
		}

		myComponent.setMarkedEdge(null);
		myComponent.clearMarkedVertices();
		myTool = myRectangleTool;
		myTool.mousePressed(e);
	}

	public void mouseReleased(MouseEvent e) {
		myTool.mouseReleased(e);
	}
	
	public void mouseClicked(MouseEvent e) {
		if ((e.getClickCount() == 2)
				&& ((e.getModifiersEx() &
    	    		MouseEvent.CTRL_DOWN_MASK) != 
    			        MouseEvent.CTRL_DOWN_MASK)) {
			Set<ISelectableVertex> vertices = 
				myComponent.getGraph().getVertices();
		
			for (ISelectableVertex v : vertices) {
				int x = e.getX();
				int y = e.getY();
				
		    	if (VertexDataCalculator.isInVertex(
		    			myComponent.getGraphics(), 
		    			v, 
		    			x, 
		    			y
		    		)
    		    	&& !(v.isParent()
		         		&& ((StarVertex) v).getChildrenCreatorRectangle(
		    	   	    	myComponent.getGraphics()
		    	        	).contains(e.getX(), e.getY())
		    	    	)
		     	) {
		    		myComponent.editLabel(v);
					return;
				}
		    	
		    	Set<IEdge> edges = v.getEdges();
		    	
		    	for (IEdge edge : edges) {
		    		if ((edge.getStart().equals(v))
		    			    && ((EdgeDataCalculator.isInEdgeLabel(
		    			    		myComponent.getGraphics(),
		    			    		edge, 
		    			    		x,
									y
								))
								|| EdgeDataCalculator.isInEdge(
			    			    		myComponent.getGraphics(),
			    			    		edge, 
			    			    		x,
										y
						    		)
						    	)) {
		    			myComponent.editLabel(edge);
		    			return;
		    		}
		    	}
			}
		}
	}
	
	public void mouseDragged(MouseEvent e){
		myTool.mouseDragged(e);
	}
	
	public void mouseMoved(MouseEvent e) {
		myTool.mouseMoved(e);
	}

	public void mouseEntered(MouseEvent e) {
		myTool.mouseEntered(e);
	}

	public void mouseExited(MouseEvent e) {
		myTool.mouseExited(e);
	}
	
	public void keyPressed(KeyEvent e){
		myTool.keyPressed(e);
	}

	public void keyReleased(KeyEvent e){
		myTool.keyPressed(e);
	}

	public void keyTyped(KeyEvent e){
		myTool.keyPressed(e);
	}
}