package ru.amse.smyshlyaev.grapheditor.ui.tool.defaulttool;

import java.awt.event.*;
import java.awt.Graphics;
import java.util.List;
import java.util.*;

import ru.amse.smyshlyaev.grapheditor.graph.Edge;
import ru.amse.smyshlyaev.grapheditor.graph.ISelectableVertex;
import ru.amse.smyshlyaev.grapheditor.ui.JGraphComponent;
import ru.amse.smyshlyaev.grapheditor.ui.tool.ITool;
import ru.amse.smyshlyaev.grapheditor.starvertex.*;


/*package*/ class DefaultVertexTool extends VertexMarker
                      implements ITool {
	private ISelectableVertex myVertex;
	private boolean myShouldClearVertices;
	private boolean myShouldRemoveVertex;
	private List<Integer> myShiftsX;
	private List<Integer> myShiftsY;

	public DefaultVertexTool(JGraphComponent component) {
		super(component);

		myVertex = null;
		myShouldClearVertices = false;
		myShouldRemoveVertex = false;
		myShiftsX = new ArrayList<Integer>();
		myShiftsY = new ArrayList<Integer>();
	}

	public void paint(Graphics gc) {
		super.paint(gc);
	}

	public ISelectableVertex getVertex() {
		return myVertex;
	}

	public void setVertex(ISelectableVertex v) {
		myVertex = v;
	}

	private void setShifts(int x, int y) {
		List<ISelectableVertex> vertices = 
			myComponent.getMarkedVertices();
		
		myShiftsX.clear();
		myShiftsY.clear();
		
		for (ISelectableVertex v : vertices) {
			myShiftsX.add(x - v.getX());
			myShiftsY.add(y - v.getY());
		}		
	}
	
	public void mousePressed(MouseEvent e) {
		if (myVertex.isParent()
			&& ((StarVertex) myVertex).getChildrenCreatorRectangle(
		    		myComponent.getGraphics()
				).contains(e.getX(), e.getY())
		) {
			Child vertex = 
				new Child(
					(StarVertex) myVertex, 
					myComponent.getGraphics()
				);			

			myComponent.addVertex(vertex);
			myComponent.addEdge(new Edge(myVertex, vertex));
		}

		setShifts(e.getX(), e.getY());
		
		if ((e.getModifiersEx() &
	    		MouseEvent.CTRL_DOWN_MASK) != 
			        MouseEvent.CTRL_DOWN_MASK) {
	    	if (myComponent.getMarkedVertices().contains(
	    			myVertex
	    		)) {
    	        myShouldClearVertices = true;
    	        return;
    	    } else {
    	    	myComponent.clearMarkedVertices();
    	    }
	    } else {
	    	if (myComponent.getMarkedVertices().contains(
	    			myVertex
        		)) {
	    		myShouldRemoveVertex = true;
	    		return;
	    	}
	    }
	    
	    if (myVertex.shouldSelectAlone()) {
	    	myComponent.clearMarkedVertices();
	    }
	    
	    for (ISelectableVertex v : myComponent.getMarkedVertices()) {
	    	if (v.shouldSelectAlone()) {
	    		myComponent.clearMarkedVertices();
	    	}
	    }
	    
		myComponent.addMarkedVertex(myVertex);
		
		setShifts(e.getX(), e.getY());
		        
    	myComponent.repaint();
	}

	public void mouseReleased(MouseEvent e) {
		if (myShouldRemoveVertex) {
			myComponent.clearMarkedVertex(myVertex);
	    	myComponent.repaint();
			myShouldRemoveVertex = false;
			
			return;
		}
		
		if (myShouldClearVertices) {
			myComponent.clearMarkedVertices();
			myComponent.addMarkedVertex(myVertex);
	    	myComponent.repaint();

			myShouldClearVertices = false;
		}
	}

	public void mouseDragged(MouseEvent e) {
		int x = e.getX();
		int y = e.getY();
		int counter = 0;

		myShouldClearVertices = false;
		myShouldRemoveVertex = false;
		
		for (ISelectableVertex v : myComponent.getMarkedVertices()) {
			v.setLocation(
					x - myShiftsX.get(counter),
					y - myShiftsY.get(counter)
            );
			
			counter += 1;
		}

		myComponent.repaint();
	}
	
	public void keyPressed(KeyEvent e) {
		System.out.println("1");
		
		switch (e.getKeyCode()) {
		case KeyEvent.VK_ESCAPE:
			myComponent.clearMarkedVertices();			
			myComponent.repaint();
			
			break;
		case KeyEvent.VK_DELETE:
			break;
		}
	}
}